#include <Arduino.h>
#include <Wire.h>
#include "SparkFun_VL53L1X.h"
#include "Sharp.h"
#include <HardwareSerial.h>
#include <U8xLaser.h>
#include <iostream>
#include <motor.h>
#include <Wifi.h>
#include "SD_write.h"
#include "FS.h"
#include "SD.h"
#include "SPI.h"
#include "stdio.h"


// Motor Pin Definitions
#define C_RESET 13
#define STP_Pl 4
#define STP_SK 26
#define DIR_SK 33
#define DIR_A 14
#define STP_A 25
#define STP_CH 32
#define DIR_CH 27
#define REVOLUTIONS 200
// Analog Sensor Pin
const byte A_sensorPin = 35;

// Chinese Sensor UART Sensor
HardwareSerial *MySerial = &Serial2;
U8xLaser Laser1 = U8xLaser(*MySerial,C_RESET);
SFEVL53L1X distanceSensor;

// Wifi Setup
#define ANALOG_S 1
#define CHINESE 2
#define SPARK 3
#define DEMO 4
const char* ssid = "SDNet";
const char* password = "CapstoneProject";

// Set web server port number to 80
WiFiServer server(80);

// Variable to store the HTTP request
String header;

String outputAnalogState = "off";
String outputChineseState = "off";
String outputSparkState = "off";
String outputDemoState = "off";

IPAddress local_IP(192,168,10,69);
IPAddress gateway(192,168,10,1);
IPAddress subnet(255,255,0,0);
IPAddress dns(192,168,1,1);

// Current time
unsigned long currentTime = millis();
// Previous time
unsigned long previousTime = 0; 
// Define timeout time in milliseconds (example: 2000ms = 2s)
const long timeoutTime = 10000;

// SD Card 
using namespace std;




void RunScan(int sensor_type);

void setup() {
  Wire.begin();

  Serial.begin(115200);
  delay(2000);
  uint32_t baudrate = 19200;
  Laser1.reset();
  Laser1.begin(baudrate);

  Serial.println("VL53L1X Qwiic Test");

  if (distanceSensor.begin() != 0) //Begin returns 0 on a good init
  {
    Serial.println("Sensor failed to begin. Please check wiring. Freezing...");
    while (1)
      ;
  }

  configMotors_raise(STP_CH,DIR_CH);
  configMotors_raise(STP_A,DIR_A);
  configMotors_raise(STP_SK, DIR_SK);
  configMotors_rotate(STP_Pl);


  Serial.println("Sensor online!");

    if (WiFi.config(local_IP,gateway,subnet,dns,dns) == false)
  {
    Serial.println("Configuration failed");
  }

  // Connect to Wi-Fi network with SSID and password
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid,password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  // Print local IP address and start web server
  Serial.println("");
  Serial.println("WiFi connected.");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
  server.begin();


  Serial.print("SD Card \n\r");
  // SD Card Setup
  if(!SD.begin(5)){
    Serial.println("Card Mount Failed");
    return;
  }
  uint8_t cardType = SD.cardType();

  if(cardType == CARD_NONE){
    Serial.println("No SD card attached");
    return;
  }

  Serial.print("SD Card Type: ");
  if(cardType == CARD_MMC){
    Serial.println("MMC");
  } else if(cardType == CARD_SD){
    Serial.println("SDSC");
  } else if(cardType == CARD_SDHC){
    Serial.println("SDHC");
  } else {
    Serial.println("UNKNOWN");
  }

  uint64_t cardSize = SD.cardSize() / (1024 * 1024);
  Serial.printf("SD Card Size: %lluMB\n", cardSize);

  deleteFile(SD,"/scan.txt");

}

void loop() {
 
  int sensor_type;

  WiFiClient client = server.available();   // Listen for incoming clients

  if (client) {                             // If a new client connects,
    currentTime = millis();
    previousTime = currentTime;
    Serial.println("New Client.");          // print a message out in the serial port
    String currentLine = "";                // make a String to hold incoming data from the client
    while (client.connected() && currentTime - previousTime <= timeoutTime) {  // loop while the client's connected
      currentTime = millis();
      sensor_type = 0;
        if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        header += c;
        if (c == '\n') {                    // if the byte is a newline character
          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (currentLine.length() == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println("Connection: close");
            client.println();
            
            // turns the GPIOs on and off
            if (header.indexOf("GET /26/on") >= 0) {
              sensor_type = CHINESE;
              outputAnalogState = "on";
              Serial.println("Analog Button");
            } else if (header.indexOf("GET /26/off") >= 0) {
              outputAnalogState = "off";
            } else if (header.indexOf("GET /27/on") >= 0) {
              sensor_type = SPARK;
              outputChineseState = "on";
              Serial.println("Chinese Button");
            }else if (header.indexOf("GET /27/off") >= 0) {
              outputChineseState = "off";
            }
            else if (header.indexOf("GET /4/on") >= 0) {
              sensor_type = ANALOG_S;
              outputSparkState = "on";
              Serial.println("Spark Button");
            }
            else if (header.indexOf("GET /4/off") >= 0) {
              outputSparkState = "off";
            }
            else if(header.indexOf("GET /25/on") >= 0){
              sensor_type = DEMO;
              outputDemoState = "on";
            }
            else if(header.indexOf("GET /25/off") >= 0 ){
              outputDemoState = "off";
            }
          
           
            // Display the HTML web page
            client.println("<!DOCTYPE html><html>");
            client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
            client.println("<link rel=\"icon\" href=\"data:,\">");
            // CSS to style the on/off buttons 
            // Feel free to change the background-color and font-size attributes to fit your preferences
            client.println("<style>html { font-family: Helvetica; display: inline-block; margin: 0px auto; text-align: center;}");
            client.println(".button { background-color: #4CAF50; border: none; color: white; padding: 16px 40px;");
            client.println("text-decoration: none; font-size: 30px; margin: 2px; cursor: pointer;}");
            client.println(".button2 {background-color: #555555;}");
            client.println(".pad {margin-bottom: 50px;}");
            client.println(".box {width: 200px; background-color: #f0f0f0; padding-top: 1%; padding-bottom: 1%; padding-left: 2%; padding-right: 2%; margin: auto; border-radius: 10px;}</style></head>");
                        
            // Web Page Heading
            client.println("<body><h1>3D Object Scanner</h1>");

            // Box to enclose all shit
            client.println("<div class=\"box\">");
            
            // Display current state, and ON/OFF buttons for GPIO 26  
            client.println("<p><b>High Quality Scan</b></p>");
            // If the output26State is off, it displays the ON button       
            if (outputAnalogState =="off") {
              client.println("<p><a href=\"/26/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/26/off\"><button class=\"button button2\">OFF</button></a></p>");
            } 
               
            // Display current state, and ON/OFF buttons for GPIO 27  
            client.println("<p><b>Medium Quality Scan</b></p>");
            // If the output27State is off, it displays the ON button       
            if (outputChineseState=="off") {
              client.println("<p><a href=\"/27/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/27/off\"><button class=\"button button2\">OFF</button></a></p>");
            }

            // Display current state, and ON/OFF buttons for LED on Pin  4  
            client.println("<p><b>Low Quality Scan</b></p>");
            // If the output4State is off, it displays the ON button       
            if (outputSparkState =="off") {
              client.println("<p><a href=\"/4/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/4/off\"><button class=\"button button2\">OFF</button></a></p>");
            } 

            client.println("<p><b>Demo Day Scan</b></p>");
            // If the output4State is off, it displays the ON button       
            if (outputDemoState =="off") {
              client.println("<p><a href=\"/25/on\"><button class=\"button\">ON</button></a></p>");
            } else {
              client.println("<p><a href=\"/25/off\"><button class=\"button button2\">OFF</button></a></p>");
            } 

            client.println("</div>");
               
            client.println("</body></html>");
            
            // The HTTP response ends with another blank line
            client.println();
            // Break out of the while loop
            break;
          } else { // if you got a newline, then clear currentLine
            currentLine = "";
          }
        } else if (c != '\r') {  // if you got anything else but a carriage return character,
          currentLine += c;      // add it to the end of the currentLine
        }
      }
    }
    // Clear the header variable
    header = "";
    // Close the connection
    client.stop();
    Serial.println("Client disconnected.");
    Serial.println("");

    // sensor_type = 4;
    RunScan(sensor_type);
    sensor_type = 0;
  }
  delay (50);
}

void RunScan(int sensor_type){
  int M_dist = 0; 
  int h_counter = 0; 
  int r_counter = 0; 
switch (sensor_type){     
    case ANALOG_S:
        while(M_dist < (REVOLUTIONS/2)){
          unsigned vin = analogRead(A_sensorPin);
          float v = float(vin)/4096 * 3.3;
          float distance2 = getDist(v);
          if (distance2 == 1000){
            M_dist++;
            distance2 = 10000;
          }
          if (r_counter == REVOLUTIONS){
            digitalWrite(DIR_A,LOW);
            for (int i = 0; i < 1600; i++){
              raiseSensor(STP_A);
            }
            r_counter = 0;
            M_dist = 0;
            h_counter++;
          }
          else {
          rotatePlatform(STP_Pl);
          r_counter++;
          }
          char result[8]; // Buffer big enough for 7-character float
            dtostrf(distance2, 6, 2, result); // Leave room for too large numbers!
            appendFile(SD, "/scan.txt", result);
            appendFile(SD,"/scan.txt",", ");
            char result2[8]; // Buffer big enough for 7-character float
            dtostrf(h_counter, 6, 2, result2); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result2);
            appendFile(SD,"/scan.txt",", ");
            char result3[8]; // Buffer big enough for 7-character float
            dtostrf(r_counter, 6, 2, result3); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result3);
            appendFile(SD, "/scan.txt", "\r\n");
            delay(100);
        }
        lowerSensorAnalog(STP_A,DIR_A, h_counter);
      break;
    case CHINESE :
        
        while(M_dist < (REVOLUTIONS/2)){
         uint16_t dist = Laser1.measureSingle();
          if (dist >= 400){
            M_dist++;
            dist = 10000;
          }
          if (r_counter == REVOLUTIONS){
            digitalWrite(DIR_CH, LOW);
            for (int i = 0; i < 1600; i++){
              raiseSensor(STP_CH);
            }
            r_counter = 0;
            M_dist = 0;
            h_counter++;
          }
          else {
            rotatePlatform(STP_Pl);
          r_counter++;
          }
          char result[8]; // Buffer big enough for 7-character float
            dtostrf(dist, 6, 2, result); // Leave room for too large numbers!
            appendFile(SD, "/scan.txt", result);
            appendFile(SD,"/scan.txt",", ");
            char result2[8]; // Buffer big enough for 7-character float
            dtostrf(h_counter, 6, 2, result2); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result2);
            appendFile(SD,"/scan.txt",", ");
            char result3[8]; // Buffer big enough for 7-character float
            dtostrf(r_counter, 6, 2, result3); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result3);
            appendFile(SD, "/scan.txt", "\r\n");
            delay(550);
        }
        lowerSensorChinese(STP_CH,DIR_CH, h_counter);
      break;
    case SPARK :
       while(M_dist < (REVOLUTIONS/2)){
          distanceSensor.startRanging(); //Write configuration bytes to initiate measurement
          while (!distanceSensor.checkForDataReady())
            {delay(1);}
          int distance = distanceSensor.getDistance(); //Get the result of the measurement from the sensor
          distanceSensor.clearInterrupt();
          distanceSensor.stopRanging();
          if (distance >= 350){
            M_dist++;
            distance = 10000;
          }
          if (r_counter == REVOLUTIONS){
            Serial.println("Raise");
            digitalWrite(DIR_SK, LOW);
            for (int i = 0; i < 1600; i++){
              raiseSensor(STP_SK);
            }
            r_counter = 0;
            M_dist = 0;
            h_counter++;
            
          }
          else {
          rotatePlatform(STP_Pl);
          r_counter++;
          }
            Serial.print(distance);
            Serial.println();
            char result[8]; // Buffer big enough for 7-character float
            dtostrf(distance, 6, 2, result); // Leave room for too large numbers!
            appendFile(SD, "/scan.txt", result);
            appendFile(SD,"/scan.txt",", ");
            char result2[8]; // Buffer big enough for 7-character float
            dtostrf(h_counter, 6, 2, result2); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result2);
            appendFile(SD,"/scan.txt",", ");
            char result3[8]; // Buffer big enough for 7-character float
            dtostrf(r_counter, 6, 2, result3); // Leave room for too large numbers!
            appendFile(SD,"/scan.txt",result3);
            appendFile(SD, "/scan.txt", "\r\n");
            delay(20);
        }
        lowerSensorSpark(STP_SK,DIR_SK, h_counter);
        break;
      case DEMO:
        int k = 1;
        while (k <= 5){
          for(int x = 0; x<10; x++){
            rotatePlatform(STP_Pl);
            r_counter++;
          }
          for (int y=0; y<1600; y++){
          raiseSensor(STP_CH);
          }
          h_counter++;

        }
        lowerSensorAnalog(STP_A,DIR_A,h_counter);
      break; 
    // default :
    //   delay(1);
  }
}
